@@ -73,11 +73,14 @@
 /* #undef HAVE_JASPER */
 
 /* AVIF codec */
-/* #undef HAVE_AVIF */
+#define HAVE_AVIF
 
 /* IJG JPEG codec */
 #define HAVE_JPEG
 
+/* JPEG XL codec */
+/* #undef HAVE_JPEGXL */
+
 /* GDCM DICOM codec */
 /* #undef HAVE_GDCM */
 
@@ -126,7 +129,7 @@
 
 /* Define if your processor stores words with the most significant byte
    first (like Motorola and SPARC, unlike Intel and VAX). */
-/* #undef WORDS_BIGENDIAN */
+#define WORDS_BIGENDIAN
 
 /* VA library (libva) */
 /* #undef HAVE_VA */
